@@ -5,16 +5,17 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-from urllib import quote
-
 from gi.repository import Gtk
 
+from quodlibet import _
 from quodlibet import config
 from quodlibet.util import website
 from quodlibet.qltk.entry import Entry
+from quodlibet.qltk import Icons
 from quodlibet.plugins.songsmenu import SongsMenuPlugin
+from quodlibet.compat import quote
 
-WIKI_URL = "http://%s.wikipedia.org/wiki/"
+WIKI_URL = "https://%s.wikipedia.org/wiki/Special:Search/"
 
 
 def get_lang():
@@ -26,7 +27,7 @@ def set_lang(value):
 
 
 class WikiSearch(object):
-    PLUGIN_ICON = Gtk.STOCK_OPEN
+    PLUGIN_ICON = Icons.APPLICATION_INTERNET
 
     @classmethod
     def changed(self, e):
@@ -43,7 +44,7 @@ class WikiSearch(object):
         e.connect('changed', self.changed)
         hb.pack_start(
             Gtk.Label(label=_("Search at %(website)s") % {
-                "website": "http://"}),
+                "website": "https://"}),
             False, True, 0)
         hb.pack_start(e, False, True, 0)
         hb.pack_start(Gtk.Label(label=".wikipedia.org"), False, True, 0)
