@@ -1,16 +1,15 @@
+# -*- coding: utf-8 -*-
 # Copyright 2014 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-from ._audio import AudioFile
+from mutagen.aac import AAC
+
+from ._audio import AudioFile, translate_errors
 
 extensions = [".aac", ".adif", ".adts"]
-try:
-    from mutagen.aac import AAC
-except ImportError:
-    extensions = []
 
 
 class AACFile(AudioFile):
@@ -21,9 +20,12 @@ class AACFile(AudioFile):
     fill_length = True
 
     def __init__(self, filename):
-        audio = AAC(filename)
+        with translate_errors():
+            audio = AAC(filename)
         self["~#length"] = audio.info.length
         self["~#bitrate"] = int(audio.info.bitrate / 1000)
+        if audio.info.channels:
+            self["~#channels"] = audio.info.channels
         self.sanitize(filename)
 
     def write(self):
@@ -42,5 +44,5 @@ class AACFile(AudioFile):
             return k == "title"
 
 
-info = AACFile
+loader = AACFile
 types = [AACFile]
